function [Ahat] = MMCROSSXY(DX,DY)
% The MM estimator based on matching the cross covariance between X and Y
% In "Improved Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019

% Inputs: 
% DX: N*dx the data of X 
% DY: N*dy the data of Y 
% Outputs:
% Ahat: dx*dy the method of moment estimator of A

% Idea: Find A such that the population moment is close to the sample moment

% Prepare for optimization
[N,dx] = size(DX);
[~,dy] = size(DY);
SigmaXYhat = DX'*DY/N; % Sample moment
[X,~,IXC] = unique(DX,'rows'); % Kx*dx the support of X, i.e., Kx mass points 
[Y,~,IYC] = unique(DY,'rows'); % Ky*dy the support of Y, i..e, Ky mass points
MX = tabulate(IXC);
MY = tabulate(IYC);
P = MX(:,3)/100; % Kx*1 the pmf vector of DX
Q = MY(:,3)/100; % Ky*1 the pmf vector of DY
SigmaXY = @(A) X'*OptimalPI(A,X,Y,P,Q)*Y; % Population moment
objf1 = @(A) sum(sum((SigmaXY(A) - SigmaXYhat).^2));

% Paramters for Optimization
warning off all
startA1 = eye(dx);
startA2 = ones(dx,dy);
SigmaXXhat = DX'*DX/N;
SigmaYYhat = DY'*DY/N;
startA3 = SigmaXXhat^(-1)*SigmaXYhat*(SigmaYYhat - SigmaXYhat'*SigmaXXhat^(-1)*SigmaXYhat)^(-1);
opts = optimset('Display','off','HessUpdate','dfp');
problem1 = createOptimProblem('fmincon', ...
                              'objective', @(A) objf1(A), ...
                              'x0', startA1, ...
                              'options', opts);
problem2 = createOptimProblem('fmincon', ...
                              'objective', @(A) objf1(A), ...
                              'x0', startA2, ...
                              'options', opts);
problem3 = createOptimProblem('fmincon', ...
                              'objective', @(A) objf1(A), ...
                              'x0', startA3, ...
                              'options', opts);                        
% Optimization
temp = zeros(3,1);
gs = GlobalSearch('Display','off'); 
[Ahat1,temp(1),~,~,~] = run(gs,problem1);
[Ahat2,temp(2),~,~,~] = run(gs,problem2);
[Ahat3,temp(3),~,~,~] = run(gs,problem3);
Ahat = Ahat1*(min(temp)==temp(1))+Ahat2*(min(temp)==temp(2))+Ahat3*(min(temp)==temp(3));

% SigmaXXhat = DX'*DX/N;
% SigmaYYhat = DY'*DY/N;
% temp = zeros(3,1);
% startA1 = SigmaXXhat^(-1)*SigmaXYhat*(SigmaYYhat - SigmaXYhat'*SigmaXXhat^(-1)*SigmaXYhat)^(-1);
% [Ahat1,temp(1)] = fminunc(@(A) objf1(A),startA1,opts);
% startA2 = ones(dx,dy);
% [Ahat2,temp(2)] = fminunc(@(A) objf1(A),startA2,opts);
% startA3 = eye(dx);
% [Ahat3,temp(3)] = fminunc(@(A) objf1(A),startA3,opts);
% Ahat = Ahat1*(min(temp)==temp(1))+Ahat2*(min(temp)==temp(2))+Ahat3*(min(temp)==temp(3));