function [VARA,VARAS] = VarAhat(DX,DY,Ahat)
% Calculate the variance of the MM estimator of A and its rescaled one
% In "Improved Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019

% Inputs: 
% DX: N*dx the data of X 
% DY: N*dy the data of Y 
% Ahat: dx*dy the method of moment estimator of A (unrescaled one)
% Outputs:
% VARA: dx*dy the variance estiamtor of the MM estimator of A
% VARAS: dx*dy the variance estiamtor of the rescaled one

% Idea: Write (P,Q,Ahat) as an MM estimator based on matching three sets of
% moment restrictions; calculate the Jacobian matrix and the variance of
% moment function evaluated at the truth 
% For rescaled one, apply the Delta method

% Information of the data
[N,dx] = size(DX);
[~,dy] = size(DY);
[X,~,~] = unique(DX,'rows'); % Kx*dx the support of X, i.e., Kx mass points 
[Y,~,~] = unique(DY,'rows'); % Ky*dy the support of Y, i..e, Ky mass points
Kx = length(X);
Ky = length(Y);

% Moment functions
G = zeros(N,Kx+Ky-2+dx*dy*3); % data matrix for calculating variance of moments
for i = 1:N
    for j = 1:Kx-1
        G(i,j) = prod(DX(i,:)==X(j,:));
    end
    for j = 1:Ky-1
        G(i,j+Kx-1) = prod(DY(i,:)==Y(j,:));
    end
    G(i,Kx+Ky-1:end-2*dx*dy) = reshape(DX(i,:)'*DY(i,:),1,dx*dy);
    G(i,end-2*dx*dy+1:end-dx*dy) = reshape(diag(DX(i,:).*DX(i,:)),1,dx*dy); 
    G(i,end-dx*dy+1:end) = reshape(diag(DY(i,:).*DY(i,:)),1,dx*dy);
end % including moments for calcualting the rescaled coeffcients 

% Variance of the moment functions
MM = (eye(N)-ones(N,N)/N)/(N-400);
VARG = G'*MM*G;
clear G MM;

% Jacobian matrix for the first two sets of moment restrictions
J1 = [eye(Kx+Ky-2),zeros(Kx+Ky-2,dx*dy)];
% Calculate the Jacobian matrix from the third set of moment function
[J2] =JACPQA(DX,DY,Ahat);
% Jacobian matrix at the truth
J = [J1;J2];

% % Jacobian matrix at the truth
% J1 = [eye(Kx+Ky-2),zeros(Kx+Ky-2,dx*dy)]; % Jacobian matrix for the first two sets of moment restrictions
% % Calculate the Jacobian matrix from the third set of moment function
% % Using the implicit function theorem
% [PIs] = OptimalPI(Ahat,X,Y,P,Q); % Optimal pmf evaluated at Ahat
% PIsm1 = reshape(PIs(1:Kx-1,1:Ky-1),(Kx-1)*(Ky-1),1);
% PFPPI = diag(-1./PIsm1) - 1/PIs(Kx,Ky)*ones((Kx-1)*(Ky-1),(Kx-1)*(Ky-1)) - kron(ones(Ky-1,Ky-1),diag(1./PIs(1:Kx-1,Ky)))...
%     -kron(diag(1./PIs(Kx,1:Ky-1)), ones(Kx-1,Kx-1)); % The FOC function with respect to PI-1
% PFPPQ = 1/PIs(Kx,Ky)*ones((Kx-1)*(Ky-1),Kx+Ky-2) + repmat([diag(1./PIs(1:Kx-1,Ky)),zeros(Kx-1,Ky-1)],Ky-1,1)...
%     +[zeros((Kx-1)*(Ky-1),Kx-1),kron(diag(1./PIs(Kx,1:Ky-1)),ones(Kx-1,1))]; % The FOC function with respect to PQ
% DDX = X(1:Kx-1,:) - repmat(X(Kx,:),Kx-1,1);
% DDY = Y(1:Ky-1,:) - repmat(Y(Ky,:),Ky-1,1); 
% PFPA = kron(DDY,DDX); % The FOC function with respect to A
% DD = kron(Y,X)';
% PIPm1 = [kron(eye(Ky-1),[eye(Kx-1);-ones(1,Kx-1)]);repmat(-eye(Kx-1),1,Ky-1);ones(1,(Kx-1)*(Ky-1))]; %PI with respect to PI-1
% J2 = DD*PIPm1*inv(PFPPI)*[PFPPQ,PFPA]; % Jacobian matrix for the third set of moment restrictions
% J = [J1;J2]; % Jacobian matrix at the truth
% clearvars -except VARG J dx dy

% Variance matrix of the MM estimator of (P,Q,A)
VARall = pinv(J)*VARG(1:end-2*dx*dy,1:end-2*dx*dy)*pinv(J)';
% Variance matrix of the MM estimator of A (unrescaled one)
VARA = VARall(end-dx*dy+1:end,end-dx*dy+1:end);


% Calculate the variance of the rescaled one
SscaleX=diag(sqrt(diag(cov(DX))));
SscaleY=diag(sqrt(diag(cov(DY)))); 
temp1 = kron(SscaleY,SscaleX);
temp2 = kron(SscaleY*Ahat',eye(dx))*pinv(kron(SscaleX,eye(dx))+kron(eye(dx),SscaleX));
temp3 = kron(eye(dy),SscaleX*Ahat)*pinv(kron(SscaleY,eye(dy))+kron(eye(dy),SscaleY));
DD = [temp1, temp2, temp3];
temp = zeros(3*dx*dy,length(VARG));
temp0 = pinv(J);
temp(1:dx*dy,1:end-2*dx*dy) = temp0(end-dx*dy+1:end,:);
temp(dx*dy+1:2*dx*dy,end-2*dx*dy+1:end-dx*dy) = eye(dx*dy);
temp(2*dx*dy+1:end,end-dx*dy+1:end) = eye(dx*dy);
VARAS = DD*temp*VARG*temp'*DD';







